#!/usr/bin/python3
#  coding: utf-8
#
#           remakeCntMeshDataForEasyIstr.py
#
#       cntData,meshDataがEasyISTR用に合致しているかチェック、修正する。
#         cnt:境界条件の整合性
#         msh:EGRP,NGRPのGENERATEを展開する。
#
#   24/03/18    新規作成
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os
import pyFistr

ls = "\n"

#
#  warningDialog
#-----------------
def warningDialog(title, message):
    """ 警告dialog表示"""
    dlg = Gtk.MessageDialog(
            parent=None,
            message_type=Gtk.MessageType.WARNING,
            buttons=Gtk.ButtonsType.OK)
    dlg.set_title(title)
    mess1 = message.split(ls)[0]
    mess2 = ls.join(message.split(ls)[1:])
    dlg.set_markup(mess1)
    dlg.format_secondary_text(mess2)
    dlg.run() 
    dlg.destroy()

#
#  isNumber
def isNumber(name):
    """ 最初の文字が数字かどうかチェックする"""
    if "0" <= name[0] and name[0] <= "9":
        return True
    return False

#
#  run
#--------
def run(currDir, FistrModel_msh, lines, meshHeaderData, FistrModel_cnt):
    """ cnt,meshDataがEasyISTR用になっているかどうかチェック修正する。"""
    flagCnt = remakeCnt(currDir, FistrModel_cnt)
    flagMsh, lines, meshHeaderData = remakeMsh(currDir, FistrModel_msh, lines, meshHeaderData)
    if flagCnt != 0 or flagMsh != 0:
        title = _("cnt,mshファイルのチェック")
        mess = _("修正して以下のbackupFileを作成しました。") + ls
        if flagCnt != 0:
            mess += "cnt: " + FistrModel_cnt + ".bak" + ls
        if flagMsh != 0:
            mess += "msh: " + FistrModel_msh + ".bak" + ls
        warningDialog(title, mess)
    return lines, meshHeaderData

#
#  remakeMsh
#-------------
def remakeMsh(currDir, FistrModel_msh, lines, meshHeaderData):
    """ meshDataがEasyISTR用になっているかどうかチェック修正する。
    GENERATEを解釈して展開する。"""
    #GENERATEを修正
    flag = 0
    for i in range(len(meshHeaderData)):
        header, data = meshHeaderData[i]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!EGROUP" or words[0] == "!NGROUP":
            if "GENERATE" in words:
                #GENERATEを展開する
                flag = 1
                data = createDataFromGenerate(data)
                words.remove("GENERATE")
                newHeader = ", ".join(words) + ls
                meshHeaderData[i] = [newHeader, data]
    if flag == 1:
        #ファイル作成、書き込み
        sourceFile = currDir + os.sep + FistrModel_msh
        newFile = sourceFile + ".bak"
        os.rename(sourceFile, newFile)
        lines = pyFistr.createLinesFistrModelMshHeader(meshHeaderData)
        f = open(sourceFile, "w"); f.writelines(lines); f.close()
    return flag, lines, meshHeaderData

#
#  createDataFromGenerate
def createDataFromGenerate(data):
    """ GENERATEからdataを作成する"""
    newData = []
    for i in range(0, len(data), 3):
        st, ed, inc = data[i], data[i+1], data[i+2]
        nData = [ii for ii in range(st, ed+1, inc)]
        newData += nData
    return newData

#
#  remakeCnt
#------------
def remakeCnt(currDir, FistrModel_cnt):
    """ cntFileがEasyISTR用になっているかチェック修正する"""
    boundaryEtc = ["!BOUNDARY", "!VELOCITY", "!ACCELERATION"]
    #cloadEtc = ["!CLOAD", "!FLOAD"]
    boundaryAll = ["!BOUNDARY", "!CLOAD", "!DLOAD", "!SPRING", 
                   "!FLOAD", "!VELOCITY", "!ACCELERATION", 
                   "!FIXTEMP", "!CFLUX", "!SFLUX", "!DFLUX", 
                   "!SFILM", "!SRADIATE"]
    cntFile = currDir + os.sep + FistrModel_cnt
    f = open(cntFile); lines = f.readlines(); f.close()
    cntHeaderData = pyFistr.getHeaderData(lines)
    #keyword内に複数のgrp名が存在する場合、分割する
    newHeaderData = []
    flag = 0
    for i in range(len(cntHeaderData)):
        header, data = cntHeaderData[i]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] in boundaryAll:
            flag, addHeaderData = separateHeaderIfMulti(header, data)
        else:
            addHeaderData = [[header, data]]
        newHeaderData += addHeaderData
    #keyword内のgrp名の数、順番を修正
    cntHeaderData = newHeaderData
    nStep = 0
    #BOUNDARY,BVELOCITY,ACCELERATION----
    newHeaderData = []
    for header, data in cntHeaderData:
        words = pyFistr.deleteSp(header).split(",")
        if words[0] in boundaryEtc:
            newData = []
            for line in data:
                ws = pyFistr.deleteSp(line).split(",")
                st, ed = int(ws[1]), int(ws[2])
                if st == ed:
                    newData += [line]
                else:
                    for i in range(st, ed+1, 1):
                        newLine = ", ".join([ws[0], str(i), str(i), ws[3]]) + ls
                        newData += [newLine]
                    flag = 1
        #CLOAD,FLOAD ---------
        elif words[0] in ["!CLOAD", "!FLOAD"]:
            wsList = list(map(lambda x: pyFistr.deleteSp(x).split(","), data))
            numList = list(map(lambda x: int(x[1]), wsList))
            nMax = max(numList)
            grp = wsList[0][0]
            if nMax > 3:
                #モーメント含む
                dataList = [[grp, str(i), "0.0"] for i in range(1, 7, 1)]
                if len(data) != 6:
                    flag = 1
            else:
                #荷重のみ
                dataList = [[grp, str(i), "0.0"] for i in range(1, 4, 1)]
                if len(data) != 3:
                    flag = 1
            for i in range(len(wsList)):
                n = numList[i]
                dataList[n-1][-1] = wsList[i][-1]
            newData = list(map(lambda x: ", ".join(x) + ls, dataList))
        #DLOAD>BXYZ --------------
        elif words[0] == "!DLOAD":
            wsList = list(map(lambda x: pyFistr.deleteSp(x).split(","), data))
            bdir = wsList[0][1]
            if bdir == "BX" or bdir == "BY" or bdir == "BZ":
                if len(wsList) < 3:
                    grp = wsList[0][0]
                    xyz = []
                    BList = list(map(lambda x: x[1], wsList))
                    if not "BX" in BList:
                        xyz += [[grp,"BX", "0.0"]]
                    if not "BY" in BList:
                        xyz += [[grp,"BY", "0.0"]]
                    if not "BZ" in BList:
                        xyz += [[grp,"BZ", "0.0"]]
                    wsList += xyz
                    wsList = sorted(wsList, key=lambda x: x[1])
                    newData = map(lambda x: ", ".join(x) + ls, wsList)
                    flag = 1
                else:
                    newData = data
            else:
                newData = data
        #STEP ----
        elif words[0] == "!STEP":
            stepName = pyFistr.getValName(header, "stepName")
            if stepName == "":
                header = pyFistr.setValName(header, "stepName=STEP"+str(nStep))
                nStep += 1
                flag = 1
            newData = data
        else:
            newData = data
        newHeaderData += [[header, newData]]
    if flag == 1:
        #ファイル作成、書き込み
        lines = []
        for header, data in newHeaderData:
            lines += [header]
            lines += data
        sourceName = currDir + os.sep + FistrModel_cnt
        backName = sourceName + ".bak"
        os.rename(sourceName, backName)
        f = open(sourceName, "w"); f.writelines(lines); f.close()
    return flag

#
#  separateHeaderIfMulti
def separateHeaderIfMulti(header, data):
    """ 1ヶのkeyword内に複数のgrpNameが含まれている場合、分割する"""
    addHeaderData = []
    bfGrp = ""
    spData = []
    flag = 0
    for line in data:
        words = pyFistr.deleteSp(line).split(",")
        currGrp = words[0]
        if isNumber(currGrp) == False:
            if currGrp != bfGrp:
                if len(spData) > 0:
                    addHeaderData += [[header, spData]]
                if bfGrp != "":
                    flag = 1
                bfGrp = currGrp
                spData = [line]
            else:
                spData += [line]
    if len(spData) > 0:
        addHeaderData += [[header, spData]]
    return flag, addHeaderData


if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    fileName = "cylinder.msh"
    cntFileName = "cylinder.cnt"
    currDir = os.getcwd()
    f = open(fileName); lines = f.readlines(); f.close()
    meshHeaderData = pyFistr.getHeaderNumData(lines)
    meshHeaderData = run(currDir, fileName, meshHeaderData, cntFileName)
    

